"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const archiver = require("archiver");
const streams = require("../../src/convert/streams");
const fsUtil = require("../../src/utils/fileSystemHandler");
const chai_1 = require("chai");
const path_1 = require("path");
const sinon_1 = require("sinon");
const stream_1 = require("stream");
const src_1 = require("../../src");
const convertTransaction_1 = require("../../src/convert/convertTransaction");
const transformers_1 = require("../../src/convert/transformers");
const errors_1 = require("../../src/errors");
const finalizers_1 = require("../mock/convert/finalizers");
const registry_1 = require("../mock/registry");
const kathyConstants_1 = require("../mock/registry/kathyConstants");
const common_1 = require("../../src/common");
const env = sinon_1.createSandbox();
class TestTransformer {
    toMetadataFormat(component) {
        return __awaiter(this, void 0, void 0, function* () {
            return {
                component,
                writeInfos: [{ output: '/type/file.m', source: new stream_1.Readable() }],
            };
        });
    }
    toSourceFormat(component, mergeWith) {
        return __awaiter(this, void 0, void 0, function* () {
            const output = mergeWith ? mergeWith.content || mergeWith.xml : '/type/file.s';
            return {
                component,
                writeInfos: [{ output, source: new stream_1.Readable() }],
            };
        });
    }
}
describe('Streams', () => {
    afterEach(() => env.restore());
    describe('ComponentReader', () => {
        it('should read metadata components one at a time', () => __awaiter(void 0, void 0, void 0, function* () {
            var e_1, _a;
            const reader = new streams.ComponentReader(kathyConstants_1.KATHY_COMPONENTS);
            let currentIndex = 0;
            try {
                for (var reader_1 = __asyncValues(reader), reader_1_1; reader_1_1 = yield reader_1.next(), !reader_1_1.done;) {
                    const component = reader_1_1.value;
                    chai_1.expect(component).to.deep.equal(kathyConstants_1.KATHY_COMPONENTS[currentIndex]);
                    currentIndex += 1;
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (reader_1_1 && !reader_1_1.done && (_a = reader_1.return)) yield _a.call(reader_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
            chai_1.expect(currentIndex).to.equal(kathyConstants_1.KATHY_COMPONENTS.length);
        }));
    });
    /**
     * NOTE: tests that call _transform methods must utilize Mocha.done to signal
     * when a test has finished and to pass on any assertion failures to the test
     * runner. Otherwise a test may fail but signal that it was successful.
     */
    describe('ComponentConverter', () => {
        const component = kathyConstants_1.KATHY_COMPONENTS[0];
        const transformer = new TestTransformer();
        beforeEach(() => {
            env.stub(transformers_1.MetadataTransformerFactory.prototype, 'getTransformer').returns(transformer);
        });
        it('should throw error for unexpected conversion format', (done) => {
            // @ts-ignore constructor argument invalid
            const converter = new streams.ComponentConverter('badformat', registry_1.mockRegistry);
            const expectedError = new errors_1.LibraryError('error_convert_invalid_format', 'badformat');
            converter._transform(component, '', (err) => {
                try {
                    chai_1.expect(err.message).to.equal(expectedError.message);
                    chai_1.expect(err.name).to.equal(expectedError.name);
                    done();
                }
                catch (e) {
                    done(e);
                }
            });
        });
        it('should transform to metadata format', (done) => {
            const converter = new streams.ComponentConverter('metadata', registry_1.mockRegistry);
            converter._transform(component, '', (err, data) => __awaiter(void 0, void 0, void 0, function* () {
                try {
                    chai_1.expect(err).to.be.undefined;
                    chai_1.expect(data).to.deep.equal(yield transformer.toMetadataFormat(component));
                    done();
                }
                catch (e) {
                    done(e);
                }
            }));
        });
        it('should transform to source format', (done) => {
            const converter = new streams.ComponentConverter('source', registry_1.mockRegistry);
            converter._transform(component, '', (err, data) => __awaiter(void 0, void 0, void 0, function* () {
                try {
                    chai_1.expect(err).to.be.undefined;
                    chai_1.expect(data).to.deep.equal(yield transformer.toSourceFormat(component));
                    done();
                }
                catch (e) {
                    done(e);
                }
            }));
        });
        it('should transform to source format using a merge component', (done) => {
            const newComponent = src_1.SourceComponent.createVirtualComponent({
                name: component.name,
                type: component.type,
                xml: path_1.join('path', 'to', 'another', 'kathys', 'a.kathy-meta.xml'),
            }, []);
            const mergeSet = new common_1.ComponentSet([component]);
            const converter = new streams.ComponentConverter('source', registry_1.mockRegistry, undefined, mergeSet);
            converter._transform(newComponent, '', (err, data) => __awaiter(void 0, void 0, void 0, function* () {
                try {
                    chai_1.expect(err).to.be.undefined;
                    chai_1.expect(data).to.deep.equal(yield transformer.toSourceFormat(newComponent, component));
                    done();
                }
                catch (e) {
                    done(e);
                }
            }));
        });
        describe('Transaction Finalizers', () => {
            let converter;
            let transaction;
            beforeEach(() => {
                transaction = new convertTransaction_1.ConvertTransaction();
                converter = new streams.ComponentConverter('metadata', registry_1.mockRegistry, transaction);
            });
            it('should not push a result if finalize did not return one', () => __awaiter(void 0, void 0, void 0, function* () {
                const pushStub = env.stub(converter, 'push');
                transaction.addFinalizer(finalizers_1.TestFinalizerNoResult);
                yield converter._flush((err) => chai_1.expect(err).to.be.undefined);
                chai_1.expect(pushStub.notCalled).to.be.true;
            }));
            it('should flush one result from a single transaction finalizer', () => __awaiter(void 0, void 0, void 0, function* () {
                const finalizer = new finalizers_1.TestFinalizerNoWrites();
                const pushStub = env.stub(converter, 'push');
                transaction.addFinalizer(finalizers_1.TestFinalizerNoWrites);
                yield converter._flush((err) => chai_1.expect(err).to.be.undefined);
                chai_1.expect(pushStub.calledOnce).to.be.true;
                chai_1.expect(pushStub.calledOnceWith(yield finalizer.finalize())).to.be.true;
            }));
            it('should flush multiple results from a single transaction finalizer', () => __awaiter(void 0, void 0, void 0, function* () {
                const pushStub = env.stub(converter, 'push');
                const results = yield new finalizers_1.TestFinalizerMultipleFormatsNoWrites().finalize();
                transaction.addFinalizer(finalizers_1.TestFinalizerMultipleFormatsNoWrites);
                yield converter._flush((err) => chai_1.expect(err).to.be.undefined);
                chai_1.expect(pushStub.calledTwice).to.be.true;
                chai_1.expect(pushStub.getCall(0).calledWith(results[0])).to.be.true;
                chai_1.expect(pushStub.getCall(1).calledWith(results[1])).to.be.true;
            }));
            it('should pass error to callback if a problem occurred', () => __awaiter(void 0, void 0, void 0, function* () {
                const expectedError = new Error('whoops');
                env.stub(transaction, 'executeFinalizers').throws(expectedError);
                yield converter._flush((err) => chai_1.expect(err).to.deep.equal(expectedError));
            }));
        });
    });
    describe('ComponentWriters', () => {
        const rootDestination = path_1.join('path', 'to', 'test-package');
        const relativeDestination = path_1.join('type', 'file.x');
        const fullPath = path_1.join(rootDestination, relativeDestination);
        const fsWritableMock = new stream_1.Writable();
        const readableMock = new stream_1.Readable();
        readableMock._read = () => {
            readableMock.push('hi');
            readableMock.push(null);
        };
        const chunk = {
            component: kathyConstants_1.KATHY_COMPONENTS[0],
            writeInfos: [
                {
                    output: relativeDestination,
                    source: readableMock,
                },
            ],
        };
        let pipelineStub;
        describe('StandardWriter', () => {
            const writer = new streams.StandardWriter(rootDestination);
            let ensureFile;
            let writableStub;
            beforeEach(() => {
                ensureFile = env.stub(fsUtil, 'ensureFileExists');
                pipelineStub = env.stub(streams, 'pipeline');
                writableStub = env
                    .stub(fs, 'createWriteStream')
                    .withArgs(fullPath)
                    // @ts-ignore
                    .returns(fsWritableMock);
            });
            it('should pass errors to _write callback', () => __awaiter(void 0, void 0, void 0, function* () {
                const whoops = new Error('whoops!');
                pipelineStub.rejects(whoops);
                yield writer._write(chunk, '', (err) => {
                    chai_1.expect(err.message).to.equal(whoops.message);
                    chai_1.expect(err.name).to.equal(whoops.name);
                });
            }));
            it('should perform file copies based on given write infos', () => __awaiter(void 0, void 0, void 0, function* () {
                pipelineStub.resolves();
                yield writer._write(chunk, '', (err) => {
                    chai_1.expect(err).to.be.undefined;
                    chai_1.expect(ensureFile.firstCall.args).to.deep.equal([fullPath]);
                    chai_1.expect(pipelineStub.firstCall.args).to.deep.equal([
                        chunk.writeInfos[0].source,
                        fsWritableMock,
                    ]);
                });
            }));
            it('should use extra info when available', () => __awaiter(void 0, void 0, void 0, function* () {
                pipelineStub.resolves();
                yield writer._write(chunk, '', (err) => {
                    chai_1.expect(err).to.be.undefined;
                    chai_1.expect(ensureFile.firstCall.args).to.deep.equal([fullPath]);
                    chai_1.expect(pipelineStub.firstCall.args).to.deep.equal([
                        chunk.writeInfos[0].source,
                        fsWritableMock,
                    ]);
                });
            }));
            it('should use full path of WriteInfo output if it is absolute', () => __awaiter(void 0, void 0, void 0, function* () {
                pipelineStub.resolves();
                const absolutePath = '/absolute/path';
                const formatWithAbsoluteOutput = {
                    component: kathyConstants_1.KATHY_COMPONENTS[0],
                    writeInfos: [
                        {
                            source: readableMock,
                            output: absolutePath,
                        },
                    ],
                };
                writableStub.withArgs(absolutePath).returns(fsWritableMock);
                yield writer._write(formatWithAbsoluteOutput, '', (err) => {
                    chai_1.expect(err).to.be.undefined;
                    chai_1.expect(ensureFile.firstCall.args).to.deep.equal([absolutePath]);
                    chai_1.expect(pipelineStub.firstCall.args).to.deep.equal([
                        formatWithAbsoluteOutput.writeInfos[0].source,
                        fsWritableMock,
                    ]);
                });
            }));
        });
        describe('ZipWriter', () => {
            let archive;
            let writer;
            beforeEach(() => {
                archive = archiver.create('zip', { zlib: { level: 3 } });
                env.stub(archiver, 'create').returns(archive);
                env
                    .stub(fs, 'createWriteStream')
                    .withArgs(`${rootDestination}.zip`)
                    // @ts-ignore
                    .returns(fsWritableMock);
                writer = new streams.ZipWriter();
            });
            it('should add entries to zip based on given write infos', () => __awaiter(void 0, void 0, void 0, function* () {
                writer = new streams.ZipWriter(`${rootDestination}.zip`);
                const appendStub = env.stub(archive, 'append');
                yield writer._write(chunk, '', (err) => {
                    chai_1.expect(err).to.be.undefined;
                    chai_1.expect(appendStub.firstCall.args).to.deep.equal([
                        chunk.writeInfos[0].source,
                        { name: chunk.writeInfos[0].output },
                    ]);
                });
            }));
            it('should finalize zip when stream is finished', () => __awaiter(void 0, void 0, void 0, function* () {
                const finalizeStub = env.stub(archive, 'finalize').resolves();
                yield writer._final((err) => {
                    chai_1.expect(err).to.be.undefined;
                    chai_1.expect(finalizeStub.calledOnce).to.be.true;
                });
            }));
            it('should write zip to buffer if no fs destination given', () => __awaiter(void 0, void 0, void 0, function* () {
                yield writer._write(chunk, '', (err) => {
                    chai_1.expect(err).to.be.undefined;
                });
                yield writer._final(() => {
                    chai_1.expect(writer.buffer).to.not.be.empty;
                });
            }));
            it('should pass errors to _write callback', () => __awaiter(void 0, void 0, void 0, function* () {
                const whoops = new Error('whoops!');
                env.stub(archive, 'append').throws(whoops);
                yield writer._write(chunk, '', (err) => {
                    chai_1.expect(err.message).to.equal(whoops.message);
                    chai_1.expect(err.name).to.equal(whoops.name);
                });
            }));
            it('should pass errors to _final callback', () => __awaiter(void 0, void 0, void 0, function* () {
                const whoops = new Error('whoops!');
                env.stub(archive, 'finalize').throws(whoops);
                yield writer._final((err) => {
                    chai_1.expect(err.message).to.equal(whoops.message);
                    chai_1.expect(err.name).to.equal(whoops.name);
                });
            }));
        });
    });
    describe('JsToXml', () => {
        it('should transform js object to xml string', () => {
            const xmlObj = {
                TestType: {
                    [common_1.XML_NS_KEY]: common_1.XML_NS_URL,
                    foo: 'bar',
                    many: [{ test: 'first' }, { test: 'second' }],
                },
            };
            const jsToXml = new streams.JsToXml(xmlObj);
            let expectedBody = common_1.XML_DECL;
            expectedBody += `<TestType xmlns="${common_1.XML_NS_URL}">\n`;
            expectedBody += '    <foo>bar</foo>\n';
            expectedBody += '    <many>\n';
            expectedBody += '        <test>first</test>\n';
            expectedBody += '    </many>\n';
            expectedBody += '    <many>\n';
            expectedBody += '        <test>second</test>\n';
            expectedBody += '    </many>\n';
            expectedBody += '</TestType>\n';
            chai_1.expect(jsToXml.read().toString()).to.be.equal(expectedBody);
        });
    });
});
//# sourceMappingURL=streams.test.js.map