"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const registry_1 = require("../../mock/registry");
const defaultMetadataTransformer_1 = require("../../../src/convert/transformers/defaultMetadataTransformer");
const path_1 = require("path");
const sinon_1 = require("sinon");
const readables_1 = require("../../mock/convert/readables");
const chai_1 = require("chai");
const common_1 = require("../../../src/common");
const src_1 = require("../../../src");
const geneConstants_1 = require("../../mock/registry/geneConstants");
const env = sinon_1.createSandbox();
describe('DefaultMetadataTransformer', () => {
    const transformer = new defaultMetadataTransformer_1.DefaultMetadataTransformer(registry_1.mockRegistry);
    beforeEach(() => env
        .stub(src_1.VirtualTreeContainer.prototype, 'stream')
        .callsFake((fsPath) => new readables_1.TestReadable(fsPath)));
    afterEach(() => env.restore());
    describe('toMetadataFormat', () => {
        it('should create a WriteInfo for each file in the component', () => __awaiter(void 0, void 0, void 0, function* () {
            const component = registry_1.simon.SIMON_COMPONENT;
            const { directoryName } = component.type;
            const relativeBundle = path_1.join(directoryName, path_1.basename(registry_1.simon.SIMON_BUNDLE_PATH));
            const expectedInfos = [];
            for (const source of component.walkContent()) {
                expectedInfos.push({
                    source: component.tree.stream(source),
                    output: path_1.join(relativeBundle, path_1.basename(source)),
                });
            }
            expectedInfos.push({
                source: component.tree.stream(component.xml),
                output: path_1.join(relativeBundle, registry_1.simon.SIMON_XML_NAME),
            });
            const result = yield transformer.toMetadataFormat(component);
            chai_1.expect(result).to.deep.equal({
                component,
                writeInfos: expectedInfos,
            });
        }));
        it('should strip the -meta.xml suffix for components with no content', () => __awaiter(void 0, void 0, void 0, function* () {
            const component = src_1.SourceComponent.createVirtualComponent(registry_1.gene.GENE_COMPONENT, []);
            const { directoryName } = component.type;
            const fileName = `${component.fullName}.${component.type.suffix}`;
            const expectedInfos = [
                {
                    output: path_1.join(directoryName, fileName),
                    source: component.tree.stream(component.xml),
                },
            ];
            chai_1.expect(yield transformer.toMetadataFormat(component)).to.deep.equal({
                component,
                writeInfos: expectedInfos,
            });
        }));
        it('should handle folder type components with no content', () => __awaiter(void 0, void 0, void 0, function* () {
            const component = src_1.SourceComponent.createVirtualComponent(registry_1.kathy.KATHY_COMPONENTS[0], []);
            const fullNameParts = component.fullName.split('/');
            const { directoryName } = component.type;
            const expectedInfos = [
                {
                    output: path_1.join(directoryName, fullNameParts[0], `${fullNameParts[1]}.${component.type.suffix}`),
                    source: component.tree.stream(component.xml),
                },
            ];
            chai_1.expect(yield transformer.toMetadataFormat(component)).to.deep.equal({
                component,
                writeInfos: expectedInfos,
            });
        }));
    });
    describe('toSourceFormat', () => {
        it('should create a WriteInfo for each file in the component', () => __awaiter(void 0, void 0, void 0, function* () {
            const component = registry_1.simon.SIMON_COMPONENT;
            const { directoryName } = component.type;
            const relativeBundle = path_1.join(common_1.DEFAULT_PACKAGE_ROOT_SFDX, directoryName, path_1.basename(registry_1.simon.SIMON_BUNDLE_PATH));
            const expectedInfos = [];
            for (const source of component.walkContent()) {
                expectedInfos.push({
                    output: path_1.join(relativeBundle, path_1.basename(source)),
                    source: component.tree.stream(source),
                });
            }
            expectedInfos.push({
                output: path_1.join(relativeBundle, registry_1.simon.SIMON_XML_NAME),
                source: component.tree.stream(component.xml),
            });
            chai_1.expect(yield transformer.toSourceFormat(component)).to.deep.equal({
                component,
                writeInfos: expectedInfos,
            });
        }));
        it('should add in the -meta.xml suffix for components with no content', () => __awaiter(void 0, void 0, void 0, function* () {
            const component = src_1.SourceComponent.createVirtualComponent(registry_1.gene.GENE_MD_FORMAT_COMPONENT, []);
            const { directoryName } = component.type;
            const fileName = `${component.fullName}.${component.type.suffix}${common_1.META_XML_SUFFIX}`;
            const expectedInfos = [
                {
                    output: path_1.join('main', 'default', directoryName, fileName),
                    source: component.tree.stream(component.xml),
                },
            ];
            chai_1.expect(yield transformer.toSourceFormat(component)).to.deep.equal({
                component,
                writeInfos: expectedInfos,
            });
        }));
        it('should handle folder type components with no content', () => __awaiter(void 0, void 0, void 0, function* () {
            const component = src_1.SourceComponent.createVirtualComponent(registry_1.kathy.KATHY_MD_FORMAT_COMPONENTS[0], []);
            const fullNameParts = component.fullName.split('/');
            const { directoryName } = component.type;
            const expectedInfos = [
                {
                    output: path_1.join(common_1.DEFAULT_PACKAGE_ROOT_SFDX, directoryName, fullNameParts[0], `${fullNameParts[1]}.${component.type.suffix}${common_1.META_XML_SUFFIX}`),
                    source: component.tree.stream(component.xml),
                },
            ];
            chai_1.expect(yield transformer.toSourceFormat(component)).to.deep.equal({
                component,
                writeInfos: expectedInfos,
            });
        }));
        it('should merge output with merge component when content is a directory', () => __awaiter(void 0, void 0, void 0, function* () {
            const root = path_1.join('path', 'to', 'another', 'simons', 'a');
            const component = src_1.SourceComponent.createVirtualComponent({
                name: 'a',
                type: registry_1.mockRegistry.types.simonpegg,
                xml: path_1.join(root, 'a.js-meta.xml'),
                content: root,
            }, [
                {
                    dirPath: root,
                    children: ['a.js-meta.xml', 'b.c', 'd.e'],
                },
            ]);
            const mergeWith = registry_1.simon.SIMON_COMPONENT;
            const expectedInfos = [
                {
                    output: path_1.join(mergeWith.content, 'b.c'),
                    source: component.tree.stream(path_1.join(root, 'b.c')),
                },
                {
                    output: path_1.join(mergeWith.content, 'd.e'),
                    source: component.tree.stream(path_1.join(root, 'd.e')),
                },
                {
                    output: path_1.join(mergeWith.content, 'a.js-meta.xml'),
                    source: component.tree.stream(path_1.join(root, 'a.js-meta.xml')),
                },
            ];
            chai_1.expect(yield transformer.toSourceFormat(component, mergeWith)).to.deep.equal({
                component,
                writeInfos: expectedInfos,
            });
        }));
        it('should merge output with merge component when content is a file', () => __awaiter(void 0, void 0, void 0, function* () {
            const root = path_1.join('path', 'to', 'another', 'keanus');
            const component = src_1.SourceComponent.createVirtualComponent({
                name: 'a',
                type: registry_1.mockRegistry.types.keanureeves,
                xml: path_1.join(root, 'a.keanu-meta.xml'),
                content: path_1.join(root, 'a.keanu'),
            }, [
                {
                    dirPath: root,
                    children: ['a.keanu-meta.xml', 'a.keanu'],
                },
            ]);
            const mergeWith = registry_1.keanu.KEANU_COMPONENT;
            const expectedInfos = [
                {
                    output: mergeWith.content,
                    source: component.tree.stream(path_1.join(root, 'a.keanu')),
                },
                {
                    output: mergeWith.xml,
                    source: component.tree.stream(path_1.join(root, 'a.keanu-meta.xml')),
                },
            ];
            chai_1.expect(yield transformer.toSourceFormat(component, mergeWith)).to.deep.equal({
                component,
                writeInfos: expectedInfos,
            });
        }));
        it('should use merge component xml path', () => __awaiter(void 0, void 0, void 0, function* () {
            const mergeWith = geneConstants_1.GENE_COMPONENT;
            const component = src_1.SourceComponent.createVirtualComponent({
                name: 'a',
                type: registry_1.mockRegistry.types.genewilder,
                xml: path_1.join('path', 'to', 'another', 'genes', geneConstants_1.GENE_XML_NAME),
            }, []);
            chai_1.expect((yield transformer.toSourceFormat(component, mergeWith)).writeInfos).to.deep.contain({
                output: mergeWith.xml,
                source: component.tree.stream(component.xml),
            });
        }));
        it('should use default relative package path if merge component has no xml', () => __awaiter(void 0, void 0, void 0, function* () {
            const component = registry_1.keanu.KEANU_COMPONENT;
            const mergeWith = src_1.SourceComponent.createVirtualComponent({
                name: 'a',
                type: registry_1.mockRegistry.types.keanureeves,
            }, []);
            chai_1.expect((yield transformer.toSourceFormat(component, mergeWith)).writeInfos).to.deep.contain({
                output: component.getPackageRelativePath(component.xml, 'source'),
                source: component.tree.stream(component.xml),
            });
        }));
    });
});
//# sourceMappingURL=defaultMetadataTransformer.js.map