"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const archiver = require("archiver");
const chai_1 = require("chai");
const path_1 = require("path");
const sinon_1 = require("sinon");
const unzipper_1 = require("unzipper");
const src_1 = require("../../../src");
const staticResourceMetadataTransformer_1 = require("../../../src/convert/transformers/staticResourceMetadataTransformer");
const errors_1 = require("../../../src/errors");
const i18n_1 = require("../../../src/i18n");
const utils_1 = require("../../../src/utils");
const registry_1 = require("../../mock/registry");
const mixedContentSingleFileConstants_1 = require("../../mock/registry/mixedContentSingleFileConstants");
const tarajiConstants_1 = require("../../mock/registry/tarajiConstants");
const readables_1 = require("../../mock/convert/readables");
const common_1 = require("../../../src/common");
const env = sinon_1.createSandbox();
describe('StaticResourceMetadataTransformer', () => {
    const transformer = new staticResourceMetadataTransformer_1.StaticResourceMetadataTransformer(registry_1.mockRegistry);
    beforeEach(() => env
        .stub(src_1.VirtualTreeContainer.prototype, 'stream')
        .callsFake((fsPath) => new readables_1.TestReadable(fsPath)));
    afterEach(() => env.restore());
    describe('toMetadataFormat', () => {
        it('should rename extension to .resource for content file', () => __awaiter(void 0, void 0, void 0, function* () {
            const component = mixedContentSingleFileConstants_1.MC_SINGLE_FILE_COMPONENT;
            const { type, content, xml } = component;
            env.stub(component, 'parseXml').resolves({
                StaticResource: {
                    contentType: 'png',
                },
            });
            const expectedInfos = [
                {
                    source: component.tree.stream(content),
                    output: path_1.join(type.directoryName, `${utils_1.baseName(content)}.${type.suffix}`),
                },
                {
                    source: component.tree.stream(xml),
                    output: path_1.join(type.directoryName, path_1.basename(xml)),
                },
            ];
            chai_1.expect(yield transformer.toMetadataFormat(component)).to.deep.equal({
                component,
                writeInfos: expectedInfos,
            });
        }));
        it('should zip directory content for all supported archive mime types', () => __awaiter(void 0, void 0, void 0, function* () {
            const component = src_1.SourceComponent.createVirtualComponent(tarajiConstants_1.TARAJI_COMPONENT, tarajiConstants_1.TARAJI_VIRTUAL_FS);
            const { type, content, xml } = component;
            const archive = archiver.create('zip', { zlib: { level: 3 } });
            const archiveDirStub = env.stub(archive, 'directory');
            const archiveFinalizeStub = env.stub(archive, 'finalize');
            const parseXmlStub = env.stub(component, 'parseXml');
            env.stub(archiver, 'create').returns(archive);
            const expectedInfos = [
                {
                    source: archive,
                    output: path_1.join(type.directoryName, `${utils_1.baseName(content)}.${type.suffix}`),
                },
                {
                    source: component.tree.stream(xml),
                    output: path_1.join(type.directoryName, path_1.basename(xml)),
                },
            ];
            for (const contentType of staticResourceMetadataTransformer_1.StaticResourceMetadataTransformer.ARCHIVE_MIME_TYPES) {
                parseXmlStub.resolves({ StaticResource: { contentType } });
                chai_1.expect(yield transformer.toMetadataFormat(component)).to.deep.equal({
                    component,
                    writeInfos: expectedInfos,
                });
                chai_1.expect(archiveDirStub.calledOnceWith(content, false)).to.be.true;
                chai_1.expect(archiveFinalizeStub.calledImmediatelyAfter(archiveDirStub)).to.be.true;
                archiveDirStub.resetHistory();
            }
        }));
        it('should throw an error if content is directory but contentType is not an archive type', () => __awaiter(void 0, void 0, void 0, function* () {
            const component = src_1.SourceComponent.createVirtualComponent(tarajiConstants_1.TARAJI_COMPONENT, tarajiConstants_1.TARAJI_VIRTUAL_FS);
            const contentType = 'nonArchiveType';
            env.stub(component, 'parseXml').resolves({ StaticResource: { contentType } });
            try {
                yield transformer.toMetadataFormat(component);
            }
            catch (e) {
                chai_1.expect(e.name).to.equal(errors_1.LibraryError.name);
                chai_1.expect(e.message).to.equal(i18n_1.nls.localize('error_static_resource_expected_archive_type', [contentType, component.name]));
            }
        }));
    });
    describe('toSourceFormat', () => {
        const mockCentralDirectory = {
            files: [
                {
                    path: 'a',
                    type: 'Directory',
                    stream: () => {
                        return null;
                    },
                },
                {
                    path: 'b/c.css',
                    type: 'File',
                    stream: () => {
                        return null;
                    },
                },
            ],
        };
        it('should rename extension from .resource to a mime extension for content file', () => __awaiter(void 0, void 0, void 0, function* () {
            const component = mixedContentSingleFileConstants_1.MC_SINGLE_FILE_COMPONENT;
            const { type, content, xml } = component;
            env.stub(component, 'parseXml').resolves({
                StaticResource: {
                    contentType: 'image/png',
                },
            });
            const expectedInfos = [
                {
                    source: component.tree.stream(content),
                    output: path_1.join(common_1.DEFAULT_PACKAGE_ROOT_SFDX, type.directoryName, `${utils_1.baseName(content)}.png`),
                },
                {
                    source: component.tree.stream(xml),
                    output: path_1.join(common_1.DEFAULT_PACKAGE_ROOT_SFDX, type.directoryName, path_1.basename(xml)),
                },
            ];
            chai_1.expect(yield transformer.toSourceFormat(component)).to.deep.equal({
                component,
                writeInfos: expectedInfos,
            });
        }));
        it('should rename extension from .resource for a fallback mime extension', () => __awaiter(void 0, void 0, void 0, function* () {
            const component = mixedContentSingleFileConstants_1.MC_SINGLE_FILE_COMPONENT;
            const { type, content, xml } = component;
            env.stub(component, 'parseXml').resolves({
                StaticResource: {
                    contentType: 'application/x-javascript',
                },
            });
            const expectedInfos = [
                {
                    source: component.tree.stream(content),
                    output: path_1.join(common_1.DEFAULT_PACKAGE_ROOT_SFDX, type.directoryName, `${utils_1.baseName(content)}.js`),
                },
                {
                    source: component.tree.stream(xml),
                    output: path_1.join(common_1.DEFAULT_PACKAGE_ROOT_SFDX, type.directoryName, path_1.basename(xml)),
                },
            ];
            chai_1.expect(yield transformer.toSourceFormat(component)).to.deep.equal({
                component,
                writeInfos: expectedInfos,
            });
        }));
        it('should rename extension from .resource for an unsupported mime extension', () => __awaiter(void 0, void 0, void 0, function* () {
            const component = mixedContentSingleFileConstants_1.MC_SINGLE_FILE_COMPONENT;
            const { type, content, xml } = component;
            env.stub(component, 'parseXml').resolves({
                StaticResource: {
                    contentType: 'application/x-myspace',
                },
            });
            const expectedInfos = [
                {
                    source: component.tree.stream(content),
                    output: path_1.join(common_1.DEFAULT_PACKAGE_ROOT_SFDX, type.directoryName, `${utils_1.baseName(content)}.bin`),
                },
                {
                    source: component.tree.stream(xml),
                    output: path_1.join(common_1.DEFAULT_PACKAGE_ROOT_SFDX, type.directoryName, path_1.basename(xml)),
                },
            ];
            chai_1.expect(yield transformer.toSourceFormat(component)).to.deep.equal({
                component,
                writeInfos: expectedInfos,
            });
        }));
        it('should ignore components without content', () => __awaiter(void 0, void 0, void 0, function* () {
            const component = Object.assign({}, mixedContentSingleFileConstants_1.MC_SINGLE_FILE_COMPONENT);
            component.content = undefined;
            chai_1.expect(yield transformer.toSourceFormat(component)).to.deep.equal({
                component,
                writeInfos: [],
            });
        }));
        it('should extract an archive', () => __awaiter(void 0, void 0, void 0, function* () {
            const component = mixedContentSingleFileConstants_1.MC_SINGLE_FILE_COMPONENT;
            const { type, xml } = component;
            env.stub(component, 'parseXml').resolves({
                StaticResource: {
                    contentType: 'application/zip',
                },
            });
            env.stub(unzipper_1.Open, 'buffer').resolves(mockCentralDirectory);
            chai_1.expect(yield transformer.toSourceFormat(component)).to.deep.equal({
                component,
                writeInfos: [
                    {
                        source: null,
                        output: path_1.join(common_1.DEFAULT_PACKAGE_ROOT_SFDX, type.directoryName, 'a', 'b', 'c.css'),
                    },
                    {
                        source: component.tree.stream(xml),
                        output: path_1.join(common_1.DEFAULT_PACKAGE_ROOT_SFDX, type.directoryName, path_1.basename(xml)),
                    },
                ],
            });
        }));
        it('should work well for null contentType', () => __awaiter(void 0, void 0, void 0, function* () {
            const component = mixedContentSingleFileConstants_1.MC_SINGLE_FILE_COMPONENT;
            const { type, content, xml } = component;
            env.stub(component, 'parseXml').resolves({
                StaticResource: {
                    contentType: undefined,
                },
            });
            chai_1.expect(yield transformer.toSourceFormat(component)).to.deep.equal({
                component,
                writeInfos: [
                    {
                        source: component.tree.stream(content),
                        output: path_1.join(common_1.DEFAULT_PACKAGE_ROOT_SFDX, type.directoryName, `${utils_1.baseName(content)}.bin`),
                    },
                    {
                        source: component.tree.stream(xml),
                        output: path_1.join(common_1.DEFAULT_PACKAGE_ROOT_SFDX, type.directoryName, path_1.basename(xml)),
                    },
                ],
            });
        }));
        it('should merge output with merge component when content is archive', () => __awaiter(void 0, void 0, void 0, function* () {
            const root = path_1.join('path', 'to', 'another', 'mixedSingleFiles');
            const component = mixedContentSingleFileConstants_1.MC_SINGLE_FILE_COMPONENT;
            const mergeComponent = src_1.SourceComponent.createVirtualComponent({
                name: mixedContentSingleFileConstants_1.MC_SINGLE_FILE_COMPONENT.name,
                type: registry_1.mockRegistry.types.mixedcontentsinglefile,
                xml: path_1.join(root, mixedContentSingleFileConstants_1.MC_SINGLE_FILE_XML_NAMES[0]),
                content: path_1.join(root, 'a'),
            }, [
                {
                    dirPath: root,
                    children: ['a'],
                },
                {
                    dirPath: path_1.join(root, 'a'),
                    children: [],
                },
            ]);
            env.stub(component, 'parseXml').resolves({
                StaticResource: {
                    contentType: 'application/zip',
                },
            });
            env.stub(unzipper_1.Open, 'buffer').resolves(mockCentralDirectory);
            chai_1.expect(yield transformer.toSourceFormat(component, mergeComponent)).to.deep.equal({
                component,
                writeInfos: [
                    {
                        source: null,
                        output: path_1.join(mergeComponent.content, 'b', 'c.css'),
                    },
                    {
                        source: component.tree.stream(component.xml),
                        output: mergeComponent.xml,
                    },
                ],
            });
        }));
        it('should merge output with merge component when content is single file', () => __awaiter(void 0, void 0, void 0, function* () {
            const root = path_1.join('path', 'to', 'another', 'mixedSingleFiles');
            const component = mixedContentSingleFileConstants_1.MC_SINGLE_FILE_COMPONENT;
            const mergeComponent = src_1.SourceComponent.createVirtualComponent({
                name: mixedContentSingleFileConstants_1.MC_SINGLE_FILE_COMPONENT.name,
                type: registry_1.mockRegistry.types.mixedcontentsinglefile,
                xml: path_1.join(root, mixedContentSingleFileConstants_1.MC_SINGLE_FILE_XML_NAMES[0]),
                content: path_1.join(root, 'a'),
            }, [
                {
                    dirPath: root,
                    children: ['a'],
                },
                {
                    dirPath: path_1.join(root, 'a'),
                    children: [],
                },
            ]);
            env.stub(component, 'parseXml').resolves({
                StaticResource: {
                    contentType: 'text/plain',
                },
            });
            chai_1.expect(yield transformer.toSourceFormat(component, mergeComponent)).to.deep.equal({
                component,
                writeInfos: [
                    {
                        source: component.tree.stream(component.content),
                        output: `${mergeComponent.content}.txt`,
                    },
                    {
                        source: component.tree.stream(component.xml),
                        output: mergeComponent.xml,
                    },
                ],
            });
        }));
    });
});
//# sourceMappingURL=staticResourceMetadataTransformer.js.map